package gpl.fredy.webmail;


import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.mail.*;
import javax.mail.internet.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.net.URLEncoder;


public class LoginServlet extends HttpServlet {

    private HttpServletRequest request;
    private HttpServletResponse response;
    private PrintWriter out;
    private HttpSession session ;
    private Folder folder;

    String smtpHost;
    
    /**
       * Get the smtphost used for this session
       * @return the smtphost used for this session
       */
    public String getSmtpHost() {

	return smtpHost;
    }
    
    /**
       * Set the value of smtphost.
       * @param v  Value to assign to smtphost.
       */
    public void setSmtpHost(String  v) {
	this.smtpHost = v;

    }
       
    String mailStore;
    
    /**
       * Get the mailstore for this session
       * @return the name of the mailstore
       */
    public String getMailStore() {

	return mailStore;
    }
    
    /**
       * Set the value of host.
       * @param v  Value to assign to host.
       */
    public void setMailStore(String  v) {
	this.mailStore = v;

    }


    String protocol;
    
    /**
       * Get the Protocol used to access the mailbox.
       * @return Value of the protocol.
       */
    public String getProtocol() {

	return protocol;
    }
    
    /**
       * Set the Protocol used to access the mailbox.
       * @param v the Protocol used to access the mailbox..
       */
    public void setProtocol(String  v) {
	this.protocol = v;
    }
     
    String user;
    
    /**
       * Get the value of user.
       * @return Value of user.
       */
    public String getUser() {

	return user;
	
    }
    /**
       * Set the value of user.
       * @param v  Value to assign to user.
       */
    public void setUser(String  v) {
	this.user = v;
    }
    
    
    String password;
    
    /**
       * Get the value of password.
       * @return Value of password.
       */
    public String getPassword() {
	return password;
    }
    
    /**
       * Set the value of password.
       * @param v  Value to assign to password.
       */
    public void setPassword(String  v) {
	this.password = v;
    }



    String defaultFolder="INBOX";
    
    /**
     * Get the value of defaultFolder.
     * @return value of defaultFolder.
     */
    public String getDefaultFolder() {
	return defaultFolder;
    }
    
    /**
     * Set the value of defaultFolder.
     * @param v  Value to assign to defaultFolder.
     */
    public void setDefaultFolder(String  v) {
	this.defaultFolder = v;
    }
    
    Properties props = System.getProperties();
    
    /**
     * Get the value of props.
     * @return value of props.
     */
    public Properties getProps() {
	return props;
    }
    
    /**
     * Set the value of props.
     * @param v  Value to assign to props.
     */
    public void setProps(Properties  v) {
	this.props = v;
    }
    
    
    String errMessage;
    
    /**
     * Get the value of errMessage.
     * @return value of errMessage.
     */
    public String getErrMessage() {
	return errMessage;
    }
    
    /**
     * Set the value of errMessage.
     * @param v  Value to assign to errMessage.
     */
    public void setErrMessage(String  v) {
	this.errMessage = v;
    }
    


    int msgCount;
    
    /**
     * Get the value of msgCount.
     * @return value of msgCount.
     */
    public int getMsgCount() {
	return msgCount;
    }
    
    /**
     * Set the value of msgCount.
     * @param v  Value to assign to msgCount.
     */
    public void setMsgCount(int  v) {
	this.msgCount = v;
        session.setAttribute("msgCount",new Integer(v));
    }
    
    String loginPage;
    
    /**
     * Get the value of loginPage.
     * @return value of loginPage.
     */
    public String getLoginPage() {
	return loginPage;
    }
    
    /**
     * Set the value of loginPage.
     * @param v  Value to assign to loginPage.
     */
    public void setLoginPage(String  v) {
	this.loginPage = v;
    }
    
    private Session mailSession;
    public Session getMailSession() { return mailSession; }   

    private Store store;
    public Store getStore() { return store; }

    private boolean verifyLogin() {

	/**
	 * here we check the followings:
	 * - is the user allowed to connect
	 * - how many folders are there
	 * - create folderlist
	 * 
	 *
	 **/
	
	boolean success = true;

	props.put("mail.smtp.host", getSmtpHost() );
	mailSession = Session.getDefaultInstance(props, null);
	try {
	    store = mailSession.getStore( getProtocol() ); 	
	} catch (NoSuchProviderException nspe) { 
	    success = false;
	    setErrMessage("No such Provider : " + getProtocol() );
	}
        try {
	    store.connect( getMailStore() , getUser() , getPassword() );
	    folder = store.getFolder( getDefaultFolder() );        
            folder.open( Folder.READ_WRITE );
            setMsgCount( folder.getMessageCount() );
	} catch ( Exception exception ) {
	    success = false ;
            setErrMessage("Exception on host " + getMailStore() + " " + 
			  exception.getMessage().toString() );
	}


	return success ;
    }



    public void init(ServletConfig config)
	throws ServletException {
	super.init(config);
	setMailStore(config.getInitParameter("mailhost"));
	setSmtpHost(config.getInitParameter("smtphost"));
        setProtocol(config.getInitParameter("protocol"));
	setLoginPage(config.getInitParameter("login-page-url")); 
    }    

    public void doGet(HttpServletRequest request,
		      HttpServletResponse response)
	              throws ServletException, IOException {
	doPost(request,response);
    }

    public void doPost(HttpServletRequest request,
		       HttpServletResponse response)
	               throws ServletException, IOException {


	this.request =request;
	this.response=response;

	out = response.getWriter();
	response.setContentType("text/HTML");
        session = request.getSession(true);

	/**
	 *  set variables received by the login-page
	 *  Must: user
	 *        password
	 *  if one is not set call the Standard-Login-page login.jsp
	 *  
	 *  if these variables are not available, use default
	 *  values: mailhost
	 *          smtphost
	 *          protocol
	 **/

	if ( (request.getParameter("user")     == null ) ||
	     (request.getParameter("password") == null ) ) displayLogin();


	setUser    (request.getParameter("user"));
	setPassword(request.getParameter("password"));
	if (  request.getParameter("mailhost")     != null )  setMailStore(request.getParameter("mailhost"));
        if (  request.getParameter("smtphost")     != null )  setSmtpHost(request.getParameter("smtphost"));
        if (  request.getParameter("protocol")     != null )  setProtocol(request.getParameter("protocol"));
	
	// set session values
	session.setAttribute("protocol",getProtocol());
	session.setAttribute("mailstore",getMailStore());
	session.setAttribute("smtphost",getSmtpHost());
	session.setAttribute("user",getUser());
	session.setAttribute("password",getPassword());
	session.setAttribute("mailsession",session);

	if ( ! verifyLogin() ) displayLogin();

	loadPage("/webmail/jsp/ListMail.jsp");
	      
    }
       
	    
    private void displayLogin() { 
	loadPage(getLoginPage());
    }
    private void loadPage(String address) {
	try {
	    RequestDispatcher dispatcher = 
		getServletContext().getRequestDispatcher(address);
	    dispatcher.forward(request, response);
	} catch ( ServletException sex ) {
	    out.println(
			"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n" +
			"<html>\n" +
			"<head>\n" +
			"<title>Servlet Error</title>\n" +
			"</head>\n" +
			"<body bcolor=\"white\">\n" +
			"<h1>Servlet Error</h1>\n" +
			"<p>\n"+
			"Something went wrong. I'm not able to process your request<br>\n" +
			"Please try again with the right URL and parameters<br></p>\n" +
			"<hr><pre>\n" + sex.getMessage().toString() + "<pre></hr></body></html>");
	} catch (IOException iex) {
	    out.println(
			"<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n" +
			"<html>\n" +
			"<head>\n" +
			"<title>Login</title>\n" +
			"</head>\n" +
			"<body bgcolor=\"white\"  >\n" +
			"<center>\n" +
			"<h1>Login</h1>\n" +
			"<p>\n" +
			"<HR>\n" +
			"<FONT size=\"4\" COLOR=\"RED\" FACE=\"Helvetica, Arial, Geneva, Sans-Serif\">\n" +
			"This is a <b>restricted</b> area.<br>\n" +
			"If you do not have explicit rights to enter, please disconnect now\n" +
			"</FONT>\n" +
			"</p>\n" +
			"<HR>   \n" +
			"<form action=\"webmail/servlet/Login\" method=post>\n" +
			"<table border=\"1\" cellspacing=\"0\" cellpadding=\"3\" bordercolor=\"lightblue\" >\n" +
			"<tr><td bgcolor=\"lightblue\" bordercolor=\"blue\" align=\"left\" nowrap>User</TD>\n" +
			"<TD bgcolor=\"white\"     bordercolor=\"blue\" align=\"left\" nowrap><input name=\"user\" size=10 maxlength=20></td></TR>\n" +
			"<tr><td bgcolor=\"lightblue\" bordercolor=\"blue\" align=\"left\" nowrap>Password</TD>\n" +
			"<TD bgcolor=\"white\"     bordercolor=\"blue\" align=\"left\" nowrap><input type=password  name=\"password\" size=10 maxlength=20></td></TR> \n" +
			"</table>\n" +
			"<input type=submit value=\"Login\"><br>\n" +
			"</form>  \n" +
			"</center>\n" +
			"<hr>\n" +
			"</body>\n" +
			"</html>\n");
	}
    }
}
	





