package gpl.fredy.webmail;

import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.mail.*;
import javax.mail.internet.*;
import java.io.*;
import javax.activation.*;
import gpl.fredy.io.*;

public class SendMessage {

    Multipart multipart ;

    MimeMessage message;
    
    /**
     * Get the value of message.
     * @return value of message.
     */
    public MimeMessage getMessage() {
	return message;
    }
    
    /**
     * Set the value of message.
     * @param v  Value to assign to message.
     */
    public void setMessage(MimeMessage  v) {
	this.message = v;
    }
    
    public void setFrom(String v) {
      try {
        getMessage().setFrom(new InternetAddress(v));
      } catch (Exception e) { }
    }

    public void addRecipient(Message.RecipientType type, String v) {
      try {
        getMessage().addRecipient(type,new InternetAddress(v));
      } catch (Exception e) { }
    }
    
    public void addLot(Message.RecipientType type, String v) {
      StringTokenizer st = new StringTokenizer(v,",");    
      while (st.hasMoreTokens()) {
        try {
          addRecipient(type,st.nextToken());
        } catch (Exception e) { } 
      }
    }
  
    public void addTO(String v)  { addLot(Message.RecipientType.TO,v);  }
    public void addCC(String v)  { addLot(Message.RecipientType.CC,v);  }
    public void addBCC(String v) { addLot(Message.RecipientType.BCC,v); }
       

    String text = "";
    public void setText(String v)   { 
      try {
        text = v; 
        BodyPart messageBodyPart = new MimeBodyPart();
        messageBodyPart.setText(v);      
        multipart.addBodyPart(messageBodyPart);
      } catch (MessagingException e) { }
    }

    public String getText() { return text; }

    public void setSubject(String v) { 
     try {
       getMessage().setSubject(v); 
     } catch (MessagingException e) { }
    }

    public void addAttachement(String fileName) {
      try {
        BodyPart bodyPart = new MimeBodyPart();
        DataSource src = new FileDataSource(fileName);
        bodyPart.setDataHandler(  new DataHandler(src) );
        bodyPart.setFileName(new File(fileName).getName());
        multipart.addBodyPart(bodyPart);
      } catch (MessagingException e) { }
    }
    public boolean send() {
      boolean success = true;
      try {
        getMessage().setContent(multipart);
        getMessage().saveChanges();
        Transport.send(getMessage());
      } catch (MessagingException e) { 
        success = false; 
        setErr(e.getMessage().toString());
      }
      return success;
    }
       
    String err;
    
    /**
     * Get the value of err.
     * @return value of err.
     */
    public String getErr() {
	return err;
    }
    
    /**
     * Set the value of err.
     * @param v  Value to assign to err.
     */
    public void setErr(String  v) {
	this.err = v;
    }
    
    public void setReply(boolean all) {
	try {
	    setMessage((MimeMessage)getMessage().reply(all));       
	} catch (MessagingException mex) { }

    }

    
    public  SendMessage(Session session) {
         Message message = new MimeMessage(session);
         setMessage( (MimeMessage) message);
         multipart = new MimeMultipart();
    }

    public SendMessage(Message message) {
         setMessage( (MimeMessage) message);
         multipart = new MimeMultipart();
    }

    public static void main(String args[]) {

	String   smtpHost="localhost";
	String   rcptTo=null; // a ,-separated List of receivers
        String   from=System.getProperty("user.name");
        String   subject="(no subject)";
        String   msgBody=null;
        String   attachement=null;
        String   msgFile = null;
        Session  session = null;

	System.out.println("SendMessage Version 1.0\n" +
			   "sends Messages via E-Mail\n\n"+
			   "Parameters:\n" +
			   "-smtp  Name of SMTP-Host\n" +
			   "-from  Name of Mailsender\n" +
			   "-to    a , separated List of receivers\n" +
			   "-s     the subject of this message\n" + 
			   "-add   a comma separated list of files to attache\n" + 
			   "-msg   the text sent with this message\n" +
			   "-MSG   the file containing the text to send with this message\n");

	int i = 0;
	while ( i < args.length) {
            if (args[i].equals("-smtp")) {
                i++;
		Properties props = new Properties();	
		props.put("mail.smtp.host", args[i]);
		session = Session.getDefaultInstance(props, null);
            }
            if (args[i].equals("-from")) {
                i++;
                from = args[i];
            }
            if (args[i].equals("-to")) {
                i++;
		rcptTo = args[i];
	    }

            if (args[i].equals("-s")) {
                i++;
		subject = args[i];
	    }
            if (args[i].equals("-add")) {
                i++;
		attachement = args[i];
	    }	    
            if (args[i].equals("-msg")) {
                i++;
		msgBody = args[i];
	    }
            if (args[i].equals("-MSG")) {
                i++;
		ReadFile rf = new ReadFile(args[i]);
		msgBody = rf.getText();
	    }
	    i++;
	}

	if ( session == null ) {
	    System.out.println("need to have a SMTP-Host\nnothing sent");
	    System.exit(0);
	}
	if (rcptTo == null) {
	    System.out.println("don't know where to send\nnothing sent");
	    System.exit(0);
	}			   

	
	SendMessage sm = new SendMessage(session);

	// setting from
        sm.setFrom(from);


	// adding receipients
	StringTokenizer st  = new StringTokenizer(rcptTo,",");
	while (st.hasMoreTokens()) {
	   sm. addTO(st.nextToken());
	}


	// set subject
	sm.setSubject(subject);


	// set Message text
	if ( msgFile != null) {
	    ReadFile rf = new ReadFile(msgFile);
	    msgBody = rf.getText();
	}
	sm.setText(msgBody);

	// add the attachement
	if ( attachement != null ) {
	    StringTokenizer sta  = new StringTokenizer(attachement,",");
	    while (sta.hasMoreTokens()) {
		sm. addAttachement(sta.nextToken());
	    }
	}

	if ( sm.send() ) {
	    System.out.println("Mail sent");
	} else {
	    System.out.println("Error or unsifficient information, Mail not sent");
	}
    }
}    
